import './JSXColoringProvider.css';

export const configureLocToMonacoRange = (monaco, parser = 'babylon') => {
    switch (parser) {
        case 'babylon':
        default:
            return loc => {
                if (!loc || !loc.start) {
                    return new monaco.Range(1, 1, 1, 1);
                }
                return new monaco.Range(loc.start.line
                    , loc.start.column + 1
                    , loc.end ? loc.end.line : loc.start.line
                    , loc.end ? loc.end.column + 1 : loc.start.column + 1,
                );
            };
    }
};

export const JSXTypes = {
    JSXOpeningElement: {
        options: {
            inlineClassName: 'mtk12.Identifier.JsxOpeningElement.Identifier',
        },
    },
    JSXClosingElement: {
        options: {
            inlineClassName: 'mtk12.Identifier.JsxClosingElement.Identifier',
        },
    },
    JSXAttribute: {
        options: {
            inlineClassName: 'mtk12.Identifier.JsxAttribute.Identifier',
        },
    },
};

let monaco = null, j = null;

class JSXColoringProvider {
    constructor(monacoRef, jRef, monacoEditor, editorId ) {
        monaco = monacoRef;
        j = jRef;
        this.locToMonacoRange = configureLocToMonacoRange(monaco);
        this.editorId = editorId;
        this.monacoEditor = monacoEditor;
    }

    afterColorize(callback) {
        this.afterColorizeCallback = callback;
    }

    colorize(ast) {
        if (ast) {
            const decorators = this.createJSXElementDecorators(ast);
            for (const jsxType in JSXTypes) {
                this.createDecoratorsByType(
                    ast, jsxType, JSXTypes[jsxType].options, decorators
                );
            }
            this.JSXDecoratorIds = this.monacoEditor
                .deltaDecorations(this.JSXDecoratorIds || [], decorators
                );
            this.afterColorizeCallback &&
            this.afterColorizeCallback(
                [...this.JSXDecoratorIds],
                ast,
                JSXTypes,
                this.locToMonacoRange,
            );
        }
    }

    colorizeCode(sourceCode) {
        this.colorize(j(sourceCode));
    }

    createJSXElementDecorators(ast, decorators = []) {
        ast
            .findJSXElements()
            .forEach(p => {
                const loc = p.value.loc;
                const openingElement = p.value.openingElement;
                let elementName = null;
                if (openingElement) {
                    const oLoc = openingElement.loc;
                    elementName = openingElement.name.name;
                    decorators.push({
                        range: new monaco.Range(
                            oLoc.start.line,
                            oLoc.start.column + 1,
                            oLoc.start.line,
                            oLoc.start.column + 2
                        ),
                        options: {
                            inlineClassName: 'mtk12.Identifier.JsxElement.Bracket',
                        }
                    });
                    decorators.push({
                        range: new monaco.Range(
                            oLoc.end.line,
                            oLoc.end.column + (openingElement.selfClosing ? -1 : 0),
                            oLoc.end.line,
                            oLoc.end.column + 1
                        ),
                        options: {
                            inlineClassName: 'mtk12.Identifier.JsxElement.Bracket',
                        }
                    });
                }
                const closingElement = p.value.closingElement;
                if (closingElement) {
                    const cLoc = closingElement.loc;
                    decorators.push({
                        range: new monaco.Range(
                            cLoc.start.line,
                            cLoc.start.column + 1,
                            cLoc.start.line,
                            cLoc.start.column + 3
                        ),
                        options: {
                            inlineClassName: 'mtk12.Identifier.JsxElement.Bracket',
                        }
                    });
                    decorators.push({
                        range: new monaco.Range(
                            cLoc.end.line,
                            cLoc.end.column,
                            cLoc.end.line,
                            cLoc.end.column + 1
                        ),
                        options: {
                            inlineClassName: 'mtk12.Identifier.JsxElement.Bracket',
                        }
                    });
                }

                decorators.push({
                    range: this.locToMonacoRange(loc),
                    options: {
                        glyphMarginClassName: 'glyph.Identifier.JsxElement',
                        glyphMarginHoverMessage:
                            `JSX Element${elementName ? ': ' + elementName : ''}`
                    }
                });
            });
        return decorators;
    }

    createDecoratorsByType(ast, jsxType, _options, decorators = [], iShowHover) {
        ast.find(j[jsxType])
            .find(j.JSXIdentifier)
            .forEach(p => {
                const loc = p.value.loc;
                const options = iShowHover ?
                    {..._options, ...{hoverMessage: `(${jsxType})`}}
                    : _options;
                decorators.push({
                    range: this.locToMonacoRange(loc),
                    options
                });
            });
        return decorators;
    }

}

export default JSXColoringProvider;